@@ -584,6 +584,15 @@ gva_columns_load (GtkTreeView *view)
         gva_error_handle (&error);
         g_object_unref (client);
 
+        if (visible_columns == NULL)
+        {
+                /* Fall back to the default columns. */
+                visible_columns = g_slist_append (
+                        visible_columns, g_strdup ("favorite"));
+                visible_columns = g_slist_append (
+                        visible_columns, g_strdup ("description"));
+        }
+
         new_columns = NULL;
         for (ii = 0; ii < G_N_ELEMENTS (default_column_order); ii++)
         {
@@ -780,14 +789,15 @@ gva_columns_get_names_full (GtkTreeView *view)
          *     It might make more sense in the column_info table,
          *     with some private API for lookups. */
 
-        /* XXX Nothing to do yet.
         for (iter = names; iter != NULL; iter = iter->next)
         {
+                /* XXX Nothing to do yet.
                 const gchar *column_name = iter->data;
 
                 if (strcmp (column_name, "description") == 0)
                         columns_add_dependency (&names, "driver_status");
-        }*/
+                */
+        }
 
         return names;
 }
