@@ -21,6 +21,7 @@
 #include <string.h>
 
 #include "gva-error.h"
+#include "gva-favorites.h"
 #include "gva-mame.h"
 #include "gva-util.h"
 
@@ -877,10 +878,18 @@ db_parser_start_element (GMarkupParseContext *context,
 
         /* XXX Copied from below... */
 
-        if (element_name == intern.game)
+        if (element_name == intern.driver)
+                db_parser_start_element_driver (
+                        data, attribute_name, attribute_value, error);
+
+        else if (element_name == intern.game)
                 db_parser_start_element_game (
                         data, attribute_name, attribute_value, error);
 
+        else if (element_name == intern.input)
+                db_parser_start_element_input (
+                        data, attribute_name, attribute_value, error);
+
         else if (element_name == intern.mame)
                 db_parser_start_element_mame (
                         data, attribute_name, attribute_value, error);
@@ -1055,7 +1064,6 @@ static ParserData *
 db_parser_data_new (GvaProcess *process)
 {
         ParserData *data;
-        gint errcode;
         GError *error = NULL;
 
         data = g_slice_new0 (ParserData);
@@ -1154,7 +1162,7 @@ db_parser_exit (GvaProcess *process,
                 gva_process_get_time_elapsed (process, &time_elapsed);
 
                 g_message (
-                        "Database built in %d.%d seconds.",
+                        "Database built in %ld.%ld seconds.",
                         time_elapsed.tv_sec, time_elapsed.tv_usec / 100000);
         }
         else
@@ -1270,6 +1278,17 @@ db_function_isfavorite (sqlite3_context *context,
                 sqlite3_result_text (context, "no", -1, SQLITE_STATIC);
 }
 
+/**
+ * gva_db_init:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Opens the games database and creates the tables if they do not already
+ * exist.  If an error occurs, it returns %FALSE and sets @error.
+ *
+ * This function should be called once when the application starts.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_init (GError **error)
 {
@@ -1299,10 +1318,20 @@ fail:
         return FALSE;
 }
 
+/**
+ * gva_db_build:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Begins the lengthy process of populating the games database and returns a
+ * #GvaProcess to track it.  The database is populated by parsing detailed
+ * game information generated by MAME.  If an error occurs while starting the
+ * parsing process, it returns %NULL and sets @error.
+ *
+ * Returns: a new #GvaProcess, or %NULL
+ **/
 GvaProcess *
 gva_db_build (GError **error)
 {
-        const gchar *filename;
         GvaProcess *process;
         ParserData *data;
 
@@ -1401,6 +1430,18 @@ gva_db_build (GError **error)
         return process;
 }
 
+/**
+ * gva_db_verify_romsets:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Starts the lengthy process of verifying the integrity of the available
+ * ROM sets and returns a #GvaProcess to track it.  The results of the
+ * verification process are written to the "romset" column of the game
+ * database.  If an error occurs while starting the verification process,
+ * it returns %NULL and sets @error.
+ *
+ * Returns: a new #GvaProcess, or %NULL
+ **/
 GvaProcess *
 gva_db_verify_romsets (GError **error)
 {
@@ -1430,6 +1471,18 @@ gva_db_verify_romsets (GError **error)
         return process;
 }
 
+/**
+ * gva_db_verify_samplesets:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Starts the lengthy process of verifying the integrity of the available
+ * sample sets and returns a #GvaProcess to track it.  The results of the
+ * verification process are written to the "sampleset" column of the game
+ * database.  If an error occurs while starting the verification process,
+ * it returns %NULL and sets @error.
+ *
+ * Returns: a new #GvaProcess, or %NULL
+ **/
 GvaProcess *
 gva_db_verify_samplesets (GError **error)
 {
@@ -1459,6 +1512,16 @@ gva_db_verify_samplesets (GError **error)
         return process;
 }
 
+/**
+ * gva_db_reset:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Removes all game information from the database and restores the tables
+ * to their initial state.  If an error occurs, it returns %FALSE and sets
+ * @error.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_reset (GError **error)
 {
@@ -1477,6 +1540,18 @@ gva_db_reset (GError **error)
         return gva_db_init (error);
 }
 
+/**
+ * gva_db_execute:
+ * @sql: an SQL statement
+ * @error: return location for a #GError, or %NULL
+ *
+ * Executes the given SQL statement without returning any results.  This
+ * function is appropriate for one-off operations like "CREATE TABLE" or
+ * "BEGIN TRANSACTION".  If an error occurs, it returns %FALSE and sets
+ * @error.
+ *
+ * Returns: %TRUE on succes, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_execute (const gchar *sql,
                 GError **error)
@@ -1498,24 +1573,62 @@ gva_db_execute (const gchar *sql,
         return (errcode == SQLITE_OK);
 }
 
+/**
+ * gva_db_transaction_begin:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Convenience function begins a database transaction.  If an error occurs,
+ * it returns %FALSE and sets @error.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_transaction_begin (GError **error)
 {
         return gva_db_execute ("BEGIN TRANSACTION", error);
 }
 
+/**
+ * gva_db_transaction_commit:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Convenience function commits a database transaction.  If an error occurs,
+ * it returns %FALSE and sets @error.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_transaction_commit (GError **error)
 {
         return gva_db_execute ("COMMIT TRANSACTION", error);
 }
 
+/**
+ * gva_db_transaction_rollback:
+ * @error: return location for a #GError, or %NULL
+ *
+ * Convenience function rolls back a database transaction.  If an error
+ * occurs, it returns %FALSE and sets @error.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_transaction_rollback (GError **error)
 {
         return gva_db_execute ("ROLLBACK TRANSACTION", error);
 }
 
+/**
+ * gva_db_prepare:
+ * @sql: an SQL statement
+ * @stmt: return location for a compiled statement handle
+ * @error: return location for a #GError, or %NULL
+ *
+ * Compiles the given SQL statement and assigns the resulting statement
+ * handle to *@stmt.  If an error occurs, it returns %FALSE and sets @error.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_prepare (const gchar *sql,
                 sqlite3_stmt **stmt,
@@ -1552,6 +1665,17 @@ db_get_build_cb (gpointer user_data,
         return 0;
 }
 
+/**
+ * gva_db_get_build:
+ * @build: return location for the build ID
+ * @error: return location for a #GError, or %NULL
+ *
+ * Returns the build ID of the MAME executable from which the contents of the
+ * database was generated.  If an error occurs, it returns %FALSE and sets
+ * @error.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
 gboolean
 gva_db_get_build (gchar **build,
                   GError **error)
@@ -1575,6 +1699,13 @@ gva_db_get_build (gchar **build,
         return (errcode == SQLITE_OK);
 }
 
+/**
+ * gva_db_get_filename:
+ *
+ * Returns the abolute path of the games database.
+ *
+ * Returns: filename for the games database
+ **/
 const gchar *
 gva_db_get_filename (void)
 {
@@ -1587,10 +1718,21 @@ gva_db_get_filename (void)
         return filename;
 }
 
+/**
+ * gva_db_needs_rebuilt:
+ * 
+ * Runs a series of tests to determine whether the game database is
+ * out-of-date and needs to be rebuilt.  Examples of conditions that
+ * would cause a database rebuild are a new version of
+ * <emphasis>GNOME Video Arcade</emphasis>, a new version of MAME,
+ * database corruption, or if the user explicitly asked us to rebuild.
+ *
+ * Returns: %TRUE if a rebuild is needed, %FALSE if the database
+ *          <emphasis>seems</emphasis> okay
+ **/
 gboolean
 gva_db_needs_rebuilt (void)
 {
-        GConfClient *client;
         gchar *db_build_id = NULL;
         gchar *mame_version = NULL;
         const gchar *reason;
@@ -1639,6 +1781,18 @@ exit:
         return rebuild;
 }
 
+/**
+ * gva_db_set_error:
+ * @error: return location for a #GError, or %NULL
+ * @code: SQLite error code
+ * @message: SQLite error message, or %NULL
+ *
+ * Does nothing if @error is %NULL; if @error is non-%NULL, then *@error must
+ * be %NULL.  Converts an SQLite error code and error message to a #GError
+ * and assigns it to *@error with a domain of %GVA_SQLITE_ERROR.  If @message
+ * is %NULL, the function calls sqlite3_errcode() to obtain the error code and
+ * sqlite3_errmsg() to obtain the error message.
+ **/
 void
 gva_db_set_error (GError **error,
                   gint code,
