@@ -19,6 +19,8 @@
 /**
  * SECTION: gva-db
  * @short_description: Database Management
+ *
+ * These functions help manage the game database.
  **/
 
 #ifndef GVA_DB_H
@@ -31,6 +33,8 @@ G_BEGIN_DECLS
 
 gboolean        gva_db_init                     (GError **error);
 GvaProcess *    gva_db_build                    (GError **error);
+GvaProcess *    gva_db_verify_romsets           (GError **error);
+GvaProcess *    gva_db_verify_samplesets        (GError **error);
 gboolean        gva_db_reset                    (GError **error);
 gboolean        gva_db_execute                  (const gchar *sql,
                                                  GError **error);
