@@ -28,6 +28,7 @@
 
 /* Command Line Options */
 gboolean opt_build_database;
+gchar *opt_inspect;
 gboolean opt_version;
 gboolean opt_which_emulator;
 
@@ -130,10 +131,12 @@ gva_find_data_file (const gchar *basename)
 /**
  * gva_get_last_version:
  *
- * Returns the most recently run version of GNOME Video Arcade prior to
- * the current run.  This is used to detect GNOME Video Arcade upgrades.
+ * Returns the most recently run version of
+ * <emphasis>GNOME Video Arcade</emphasis> prior to the current run.  This
+ * is used to detect <emphasis>GNOME Video Arcade</emphasis> upgrades.
  *
- * Returns: the most recently run version of GNOME Video Arcade
+ * Returns: the most recently run version of
+ *          <emphasis>GNOME Video Arcade</emphasis>
  **/
 const gchar *
 gva_get_last_version (void)
@@ -177,7 +180,6 @@ gchar *
 gva_get_monospace_font_name (void)
 {
         GConfClient *client;
-        PangoFontDescription *desc;
         gchar *font_name;
         GError *error = NULL;
 
@@ -215,6 +217,8 @@ gva_get_time_elapsed (GTimeVal *start_time,
  *
  * Returns the directory where user-specific application data is stored.
  * The function also creates the directory the first time it is called.
+ *
+ * Returns: user-specific application data directory
  **/
 const gchar *
 gva_get_user_data_dir (void)
