@@ -23,7 +23,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifdef WITH_GNOME
+#include <gnome.h>
+#endif
+
 #include "gva-db.h"
+#include "gva-error.h"
 #include "gva-history.h"
 #include "gva-main.h"
 #include "gva-mame.h"
@@ -31,6 +36,7 @@
 #include "gva-preferences.h"
 #include "gva-properties.h"
 #include "gva-search.h"
+#include "gva-tree-view.h"
 #include "gva-ui.h"
 
 static GOptionEntry entries[] =
@@ -39,6 +45,11 @@ static GOptionEntry entries[] =
           G_OPTION_ARG_NONE, &opt_build_database,
           N_("Build the games database"), NULL },
 
+        { "inspect", 'i', 0,
+          G_OPTION_ARG_STRING, &opt_inspect,
+          N_("Inspect an emulator setting"),
+          N_("NAME") },
+
         { "version", 'v', 0,
           G_OPTION_ARG_NONE, &opt_version,
           N_("Show the application version"), NULL },
@@ -55,7 +66,6 @@ start (void)
 {
         gchar *mame_version;
         guint context_id;
-        gint last_view;
         GError *error = NULL;
 
         context_id = gva_main_statusbar_get_context_id (G_STRFUNC);
@@ -98,18 +108,52 @@ start (void)
 gint
 main (gint argc, gchar **argv)
 {
+#ifdef WITH_GNOME
+        GnomeProgram *program;
+        GOptionContext *context;
+#endif
         GError *error = NULL;
 
+        g_thread_init (NULL);
+
         bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
         textdomain (GETTEXT_PACKAGE);
 
-        g_thread_init (NULL);
+#ifdef WITH_GNOME
+        context = g_option_context_new (NULL);
+        g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
+        g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
 
+        program = gnome_program_init (
+                PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
+                GNOME_PROGRAM_STANDARD_PROPERTIES,
+                GNOME_PARAM_GOPTION_CONTEXT, context,
+                GNOME_PARAM_NONE);
+#else
         gtk_init_with_args (
                 &argc, &argv, NULL, entries, GETTEXT_PACKAGE, &error);
         if (error != NULL)
                 g_error ("%s", error->message);
+#endif
+
+        if (opt_inspect != NULL)
+        {
+                gchar *value;
+
+                value = gva_mame_get_config_value (opt_inspect, &error);
+                if (value != NULL)
+                {
+                        g_print ("%s\n", value);
+                        g_free (value);
+                }
+                else
+                {
+                        g_printerr ("%s\n", error->message);
+                        g_clear_error (&error);
+                }
+                exit (0);
+        }
 
         if (opt_version)
         {
@@ -143,5 +187,9 @@ main (gint argc, gchar **argv)
 
         gtk_main ();
 
+#ifdef WITH_GNOME
+        g_object_unref (program);
+#endif
+
         return EXIT_SUCCESS;
 }
